import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.DocumentType;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

public class DOMUtil
{
	/**
	 * Find element with a given name starting at a given node.
	 * @param startNode starting node
	 * @param elementName element name to search for
	 */
	static Node findElement( Node startNode, String elementName )
	{
		while (startNode != null)
		{
			if (startNode.getNodeType() == Node.ELEMENT_NODE &&
				startNode.getNodeName().equals( elementName ))
			{
				return startNode;
			}
			startNode = startNode.getNextSibling();
		}
		return null;
	}
	
	/**
	 * Find element with a given name among children of a given node.
	 * @param startNode starting node whose children are to be searched
	 * @param elementName element name to search for
	 */
	static Node findChildElement( Node startNode, String elementName )
	{
		startNode = startNode.getFirstChild();
		return findElement( startNode, elementName );
	}
	
	/**
	 * Find text of element with a given name starting at a given node.
	 * Finds only the first text node.
	 * @param startNode starting node whose children are to be searched
	 * @param elementName element name to search for
	 */
	static String findValue( Node startNode, String elementName )
	{
		Node n = findElement( startNode, elementName );
		if (n != null)
		{
			n = n.getFirstChild();
			while (n != null && n.getNodeType() != Node.TEXT_NODE)
			{
				n = n.getNextSibling();
			}
			if (n != null)
			{
				return n.getNodeValue();
			}
		}
		return "";
	}

	/**
	 * Find text of a given node.
	 * Finds only the first text node.
	 * @param startNode starting node whose children are to be searched
	 */
	static String findValue( Node startNode )
	{
		Node n = startNode;
		if (n != null)
		{
			n = n.getFirstChild();
			while (n != null && n.getNodeType() != Node.TEXT_NODE)
			{
				n = n.getNextSibling();
			}
			if (n != null)
			{
				return n.getNodeValue();
			}
		}
		return "";
	}
	
	/**
	 * Find text of element with a given name among children of a given node.
	 * @param startNode starting node whose children are to be searched
	 * @param elementName element name to search for
	 */
	static String findChildValue( Node startNode, String elementName )
	{
		startNode = startNode.getFirstChild();
		return findValue( startNode, elementName );
	}
	
	/**
	 * Find value of a specific attribute of a given node.
	 * @param startNode starting node whose attributes are to be searched
	 * @param attrName attribute name to search for
	 */
	static String findAttrValue( Node startNode, String attrName )
	{
		NamedNodeMap attrs = startNode.getAttributes();
		Node n;

		if (attrs != null)
		{
			n = attrs.getNamedItem( attrName );
			if (n != null)
			{
				return n.getNodeValue();
			}
		}
		return "";
	}
	
	/**
	 * Create a named element containing text.
	 * @param doc document that will contain the element
	 * @param elementName element name to create
	 * @param elementText text content of the new element
	 */
	static DocumentFragment createTextElement( 
		Document doc, String elementName, String elementText)
	{
		DocumentFragment d = doc.createDocumentFragment();
		Element e = doc.createElement( elementName );
		Node t;
		d.appendChild( e );
		t = doc.createTextNode( elementText );
		e.appendChild( t );
		d.appendChild( e );
		return d;
	}
	
	/**
	 * Replace the text content of an element
	 * @param parent element to be updated
	 * @param newText text to be inserted as child of the element
	 */
	static void replaceText( Element parent, String newText )
	{
		Document doc = parent.getOwnerDocument();
		
		if (parent.hasChildNodes())
		{
			parent.removeChild( parent.getFirstChild() );
		}
		parent.appendChild( doc.createTextNode( newText ) );
	}
	
	/**
	 * Replace the text content of a node's child element
	 * This function will update the first element of the given
	 * name that is a child of the starting node.
	 * @param startNode starting node for the search
	 * @param elementName element to be updated
	 * @param newText text to be inserted as child of the element
	 */
	static void setText( Element startNode,
		String elementName,
		String newText )
	{
		Node n = startNode;
		if (n != null)
		{
			n = n.getFirstChild();
			while (n != null)
			{
				if (n.getNodeType() == Node.ELEMENT_NODE &&
					((Element) n).getTagName().equals(elementName))
				{
					break;
				}
				n = n.getNextSibling();
			}
			if (n != null)
			{
				Node t = n.getOwnerDocument().createTextNode( newText );
				if (n.hasChildNodes())
				{
					n.removeChild( n.getFirstChild() );
				}
				n.appendChild( t );
			}
		}
	}
	
	/**
	 * Normalize whitespace in a string.
	 * Strip trailing and leading whitespace, and
	 * compress all runs of internal whitespace to single blanks.
	 * @param str input string
	 */
	 static public String normalizeSpace( String str )
	 {
	 	String			newStr;
		StringBuffer	buf = new StringBuffer( str.length() );
		int				i;
		char			c;
		
		newStr = str.trim();
		i = 0;
		while (i < newStr.length())
		{
			c = newStr.charAt( i );
			buf.append( c );
			if (Character.isWhitespace( c ))
			{
				while (Character.isWhitespace( newStr.charAt( i ) ))
				{
					i++;
				}
			}
			else
			{
				i++;
			}
		}
		return buf.toString();
	}
}
